/*********************************************************************
 *
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2011, Robert Bosch LLC.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of the Robert Bosch nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 *********************************************************************/

//\Author Adrian Funk, Bosch LLC

// This node fakes data of the proximity sensor
// This might be useful for debugging reasons with the skin

#include <ros/ros.h>
#include <proximity_sensor_driver/prox_sensor_measurement.h>
#include <proximity_sensor_driver/reinitialize_proximity_sensor.h>
#include <time.h>

// Fake the maximum amount of possible patches
const int INT_FAKED_PATCHES = 192;

// Fake callback for the reinit service
bool reinitialize_callback( proximity_sensor_driver::reinitialize_proximity_sensor::Request &req,
                            proximity_sensor_driver::reinitialize_proximity_sensor::Response &res)
{
  ROS_INFO("Reconfigure callback executed!");
  return(true);
}

int main(int argc, char **argv)
{
	srand(time(NULL));
	ros::init(argc, argv, "fake_skin_data");
	ros::NodeHandle nh;

	ros::Publisher prox_data_pub = nh.advertise<proximity_sensor_driver::prox_sensor_measurement>("prox_sensor_data", 100);
	ros::ServiceServer prox_serv = nh.advertiseService("reinitialize_proximity_sensor", reinitialize_callback);

	// Fake data in an endless loop at 15hz
	ros::Rate loop_rate(15);
	int int_count = 1;
	while(ros::ok())
	{
		// Execute Callback from reconfigure
		ros::spinOnce();
		// Build the message
		proximity_sensor_driver::prox_sensor_measurement prox_data;
		prox_data.header.stamp = ros::Time::now();
		prox_data.header.seq = int_count;
		prox_data.cell_count = INT_FAKED_PATCHES;
		prox_data.is_connected = true;
		prox_data.status_err = false;
		prox_data.status_go = true;

		for(int a = 0; a < INT_FAKED_PATCHES; a++)
		{
			prox_data.value[a] = rand() % 255;	// Create some random readings
			prox_data.offset[a] = 120;			// But keep a constant offset
			prox_data.limit[a] = 8;
		}

		prox_data_pub.publish(prox_data);

		loop_rate.sleep();
		int_count++;
	}

	return(0);
}
